'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = rStampit;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashObjectAssign = require('lodash/object/assign');

var _lodashObjectAssign2 = _interopRequireDefault(_lodashObjectAssign);

var _lodashObjectHas = require('lodash/object/has');

var _lodashObjectHas2 = _interopRequireDefault(_lodashObjectHas);

var _lodashLangIsEmpty = require('lodash/lang/isEmpty');

var _lodashLangIsEmpty2 = _interopRequireDefault(_lodashLangIsEmpty);

var _lodashObjectOmit = require('lodash/object/omit');

var _lodashObjectOmit2 = _interopRequireDefault(_lodashObjectOmit);

var _lodashObjectPick = require('lodash/object/pick');

var _lodashObjectPick2 = _interopRequireDefault(_lodashObjectPick);

var _stampit = require('stampit');

var _stampit2 = _interopRequireDefault(_stampit);

var _utils = require('./utils');

/**
 * Return a factory function (called a stamp) that will produce new
 * React components using the prototypes that are passed in or composed.
 *
 * @param {Object} React  The React library.
 * @param {Object} [props]  A map of property names and values specialized for React.
 * @return {Function} stamp A factory to produce React components using the given properties.
 * @return {Object} stamp.fixed An object map containing the fixed prototypes.
 */

function rStampit(React, props) {
  var stamp = React ? _stampit2['default'].convertConstructor(React.Component) : (0, _stampit2['default'])();
  var refs = undefined,
      init = undefined,
      methods = undefined,
      statics = undefined;

  // Shortcut for converting React's class to a stamp.
  if ((0, _lodashLangIsEmpty2['default'])(props)) {
    stamp.compose = _utils.compose;
    return (0, _utils.stripStamp)(stamp);
  }

  init = props.init || [];
  refs = props.state && { state: props.state } || {};
  statics = (0, _lodashObjectAssign2['default'])({}, props.statics, (0, _lodashObjectPick2['default'])(props, ['contextTypes', 'childContextTypes', 'propTypes', 'defaultProps']), { displayName: props.displayName || 'ReactStamp' });
  methods = (0, _lodashObjectOmit2['default'])(props, function (val, key) {
    return (0, _lodashObjectHas2['default'])(statics, key) || ['init', 'state', 'statics'].indexOf(key) >= 0;
  });

  stamp = stamp.init(init).refs(refs).methods(methods)['static'](statics);
  stamp.compose = _utils.compose;

  return (0, _utils.stripStamp)(stamp);
}

module.exports = exports['default'];