'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = compose;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashObjectAssign = require('lodash/object/assign');

var _lodashObjectAssign2 = _interopRequireDefault(_lodashObjectAssign);

var _lodashCollectionForEach = require('lodash/collection/forEach');

var _lodashCollectionForEach2 = _interopRequireDefault(_lodashCollectionForEach);

var _lodashObjectHas = require('lodash/object/has');

var _lodashObjectHas2 = _interopRequireDefault(_lodashObjectHas);

var _lodashObjectMapValues = require('lodash/object/mapValues');

var _lodashObjectMapValues2 = _interopRequireDefault(_lodashObjectMapValues);

var _lodashObjectOmit = require('lodash/object/omit');

var _lodashObjectOmit2 = _interopRequireDefault(_lodashObjectOmit);

var _stampit = require('stampit');

var _stampit2 = _interopRequireDefault(_stampit);

var _ = require('../');

var _2 = _interopRequireDefault(_);

var _3 = require('./');

var dupeFilter = function dupeFilter(prev, next, key, targ) {
  if (targ[key]) {
    throw new TypeError('Cannot mixin key `' + key + '` because it is provided by multiple sources.');
  }

  return next;
};

/**
 * React specification for creating new components
 */
var reactSpec = {
  propTypes: 'many_merged_dupe',
  defaultProps: 'many_merged_dupe',
  contextTypes: 'many_merged',
  childContextTypes: 'many_merged',
  getChildContext: 'many_merged_dupe',
  render: 'once',
  componentWillMount: 'many',
  componentDidMount: 'many',
  componentWillReceiveProps: 'many',
  shouldComponentUpdate: 'once',
  componentWillUpdate: 'many',
  componentDidUpdate: 'many',
  componentWillUnmount: 'many'
};

/**
 * Iterate through stamp methods, creating wrapper
 * functions for mixable React methods, starting
 * execution with first-in.
 *
 * @param {Object} targ Method destination
 * @param {Object} src New methods
 * @return {Object} An object of methods
 */
function wrapMethods(targ, src) {
  var methods = undefined;

  methods = (0, _lodashObjectMapValues2['default'])(src, function (val, key) {
    if (typeof val === 'function') {
      switch (reactSpec[key]) {
        case 'many':
          return function () {
            /* eslint-disable no-unused-expressions */
            targ[key] && targ[key].apply(this, arguments);
            val.apply(this, arguments);
            /* eslint-disable no-unused-expressions */
          };
        case 'many_merged_dupe':
          return function () {
            var res1 = targ[key] && targ[key].apply(this, arguments);
            var res2 = val.apply(this, arguments);

            return res1 ? (0, _lodashObjectAssign2['default'])(res1, res2, dupeFilter) : res2;
          };
        case 'once':
        default:
          if (!targ[key]) {
            return val;
          }

          throw new TypeError('Cannot mixin `' + key + '` because it has a unique constraint.');
      }
    }
  });

  return (0, _lodashObjectAssign2['default'])({}, targ, methods);
}

/**
 * Process the static properties of a stamp and
 * combine the result with the passed in statics object.
 *
 * @param {Object} stamp A stamp
 * @param {Object} prev An object of past static properties
 * @return {Object} A processed object of static properties
 */
function extractStatics(targ, src) {
  var statics = (0, _lodashObjectAssign2['default'])({}, targ);

  (0, _lodashCollectionForEach2['default'])(src, function (val, key) {
    if (reactSpec[key] === 'many_merged_dupe') {
      statics[key] = (0, _lodashObjectAssign2['default'])({}, statics[key], val, dupeFilter);
    } else if (reactSpec[key] === 'many_merged') {
      statics[key] = (0, _lodashObjectAssign2['default'])({}, statics[key], val);
    } else {
      statics[key] = val;
    }
  });

  return statics;
}

/**
 * Take two or more stamps produced from react-stampit or
 * stampit and combine them to produce and return a new stamp.
 *
 * @param {...Function} stamp Two or more stamps.
 * @return {Function} A new stamp composed from arguments.
 */

function compose() {
  var result = (0, _stampit2['default'])(),
      refs = { state: {} },
      init = [],
      methods = {},
      statics = {};

  for (var _len = arguments.length, stamps = Array(_len), _key = 0; _key < _len; _key++) {
    stamps[_key] = arguments[_key];
  }

  if ((0, _3.isStamp)(this)) stamps.push(this);

  (0, _lodashCollectionForEach2['default'])(stamps, function (stamp) {
    stamp = !(0, _3.isStamp)(stamp) ? (0, _2['default'])(null, stamp) : stamp; // eslint-disable-line

    init = init.concat(stamp.fixed.init);
    methods = wrapMethods(methods, stamp.fixed.methods);
    statics = extractStatics(statics, (0, _lodashObjectOmit2['default'])(stamp, function (val, key) {
      return (0, _lodashObjectHas2['default'])(result, key);
    }));

    if (stamp.fixed.refs && stamp.fixed.refs.state) {
      (0, _lodashObjectAssign2['default'])(refs.state, stamp.fixed.refs.state, dupeFilter);
    }
  });

  result = result.init(init).refs(refs).methods(methods)['static'](statics);
  result.compose = compose;

  return (0, _3.stripStamp)(result);
}

module.exports = exports['default'];